/*
 * Copyright (C) 2003-2007 eXo Platform SAS.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see<http://www.gnu.org/licenses/>.
 */
package org.exoplatform.services.jcr.load.blob.thread;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.load.blob.TestConcurrentItems;

/**
 * Created by The eXo Platform SAS Author : Peter Nedonosko peter.nedonosko@exoplatform.com.ua
 * 24.10.2006
 * 
 * @version $Id: DeleteThread.java 11907 2008-03-13 15:36:21Z ksm $
 */
public class DeleteThread extends UserThread {

  public DeleteThread(Session threadSession) {
    super(threadSession);
  }

  public void testAction() {
    while (process || TestConcurrentItems.consumedNodes.size() > 0) {
      deleteAction();
      try {
        sleep(2500);
      } catch (InterruptedException e) {
        threadLog.error("Sleep error: " + e.getMessage(), e);
      }
    }
  }

  public void deleteAction() {

    final String[] nodes = TestConcurrentItems.consumedNodes.toArray(new String[TestConcurrentItems.consumedNodes.size()]);
    try {
      threadSession.refresh(false);
    } catch (RepositoryException th) {
      threadLog.error("Refresh before delete error: " + th.getMessage(), th);
    }
    for (String nodePath : nodes) {
      String nodeInfo = "";
      try {
        Node node = (Node) threadSession.getItem(nodePath);
        PropertyImpl data = (PropertyImpl) node.getProperty("jcr:content/jcr:data");
        nodeInfo = "node: " + node.getPath() + ", data: " + data.getInternalIdentifier();
        node.remove();
        threadSession.save();
        if (threadLog.isDebugEnabled())
          threadLog.debug("Delete " + nodeInfo);
      } catch (PathNotFoundException e) {
        threadLog.warn(e.getMessage());
      } catch (RepositoryException e) {
        try {
          threadSession.refresh(false);
        } catch (RepositoryException e1) {
          threadLog.error("Rollback repository error: " + e1.getMessage() + ". Root exception " + e,
                          e);
        }
      } catch (Throwable th) {
        threadLog.error("Delete error: " + th.getMessage() + ". " + nodeInfo, th);
      } finally {
        TestConcurrentItems.consumedNodes.remove(nodePath);
      }
    }
  }
}
